
#include "bpol/lhca.h"
#include "bits/bitcombcolex.h"
#include "bpol/bitpolirred.h"
#include "bpol/bitpolprimitive.h"
#include "bits/revbin.h"
#include "bpol/gf2n.h"  // for factorization of mersenne prime

#include "bits/printbin.h"

#include "fxtiomanip.h"
#include "bits/bitsperlong.h"

#include <cstdlib>  // atol()


//% Generate minimal-weight low-bit LHCA rules.


ulong pn;

void
print_rules(ulong *rr, ulong *cc, ulong n)
{
    for (ulong k=0; k<n; ++k)
    {
        print_bin_nn("  r=", rr[k], pn);
        print_bin_nn("  c=", cc[k], pn);
        print_idx_seq_nn("  r= ", rr[k], 1);  // cf. data/minweight-lhca-rules.txt
//        print_idx_seq_nn("  c= ", cc[k]);
        cout << endl;
    }
}
// -------------------------


int
main(int argc, char **argv)
{
    ulong nn = BITS_PER_LONG - 1;
    if ( argc>1 )  nn = atol(argv[1]);
    pn = nn + 1;

    ulong m = 3;  // search rules of weight 1,2,...,m
    if ( argc>2 )  m = atol(argv[2]);

//    cout << " n <= "; cout.width(2); cout << nn << endl;
    cout.width(pn);  cout << "LHCA rule";
    cout.width(pn);  cout << "polynomial";
    cout << endl;

    ulong rr[999];
    ulong cc[999];

    for (ulong n=1; n<=nn; ++n)
    {
        cout.width(2);  cout << n << ":  ";
        ulong ct = 0;
        ulong h = 1UL << (n-1);
        ulong n2 = h<<1;
        GF2n::init(n);
        for (ulong nb=1; nb<=m; ++nb)
        {
            ulong r = first_comb(nb);
            do
            {
                ulong c = lhca2poly(r, n);
//                if ( (c<<1) <= h )  goto next;
//                if ( revbin(r,n)<r )  goto next;
                if ( 0==bitpol_irreducible_q(c, h) )  goto next;
                if ( 0!=test_bitpol_primitive(c, h, GF2n::mfact_ ) )  goto next;

                cc[ct] = c;
                rr[ct] = r;
                ++ct;
                goto print;  // (minimal weight rule only)

            next:
                r = next_colex_comb(r);
            }
            while ( r<n2 );
        }

    print:
        print_rules(rr, cc, ct);
    }

    return 0;
}
// -------------------------

