
#include "comb/equivclasses.h"  // equivalence_classes()
#include "sort/sortbykey.h"  // sort_by_key()

#include "bits/bitcyclic3.h"  // bit_cyclic_dist()

//#include "perm/randperm.h" // random_permute()

#include "bits/printbin.h" // print_bin_nn()
#include "demo/nextarg.h" // NXARG()

#include "fxtiomanip.h"
#include "fxtalloca.h"


//% Equivalence classes: necklaces and bracelets.


static ulong b;  // number of bits
bool n_equiv_q(ulong x, ulong y)
{
    ulong d = bit_cyclic_dist(x, y, b);
    return  (0==d);
}
// -------------------------

static ulong mm;  // mask to complement
bool b_equiv_q(ulong x, ulong y)
{
    ulong d = bit_cyclic_dist(x, y, b);
    if ( 0!=d )  d = bit_cyclic_dist(mm^x, y, b);
    return  (0==d);
}
// -------------------------


int
main(int argc, char **argv)
{
    b = 4;
    NXARG(b, "Number of bits of necklaces/bracelets.");
    ulong n = 1UL<<b;

    mm = 1;
    NXARG(mm, "Whether equivalence includes complements (==> bracelets).");
    if ( mm )  mm = n-1;

    ulong ksq = 0;
    NXARG(ksq, "Whether to print key-sorted array.");

    ALLOCA(ulong, s, n);
    for (ulong k=0; k<n; ++k)  s[k] = k;
//    random_permute(s, n);
    
    ulong *e;
    if ( 0==mm )  e = equivalence_classes(s, n, n_equiv_q);
    else          e = equivalence_classes(s, n, b_equiv_q);

    // --- print all elements with 'coset leaders':
    for (ulong k=0; k<n; ++k)
    {
        cout << setw(3) << k << ": ";
        print_bin_nn(" ", s[k], b);
        cout << "  " << setw(2) << e[k];
        cout << endl;
    }
    cout << endl;


    sort_by_key(s, n, e, 1);
    if ( ksq )
    {
        for (ulong k=0; k<n; ++k)
        {
            cout << setw(3) << k << ": ";
            print_bin_nn(" ", s[k], b);
            cout << "  " << setw(2) << e[k];
            cout << endl;
        }
        cout << endl;
    }

    // --- print all elements in each equivalence class:
    for (ulong k=0; k<n; /**/ )
    {
        ulong el = e[k];
        cout << setw(3) << el << ": ";
        ulong ct = 0;
        for (ct=0; k<n; ++ct,++k)
        {
            ulong ek = e[k];
            if ( ek != el )
            {
                break;
            }
            print_bin_nn("  ", s[k], b);
        }
        cout << "  [#=" << ct << "]";
        cout << endl;
    }
    cout << endl;

    return 0;
}
// -------------------------

