
#include "haar-all.h"
#include "walsh-all.h"
#include "perm-all.h"

#include "aux1-all.h"  // test_delta() etc.

#include "fxtiomanip.h"

#include <cstdlib>  // atol()


//% Basis matrices of Walsh and Haar transforms.


#define  TT(x)  { if ( q ) { cout << "    " << #x << endl;}  x; }


template <typename Type>
void prsgn(Type *f, ulong n, ulong x, double eps=1e-7)
{
    cout << "  ";
    cout.width(3);  cout << x << ":";
    cout << " ";

    double s1 = sign( fabs(f[0]) > eps ? f[0] : 0);
    int seq = 0;
    cout << "[ ";
    for (ulong k=0; k<n; ++k)
    {
        double v = f[k];
        if ( fabs(v) < eps )  v = 0;
        double s2 = (v > 0 ? +1 : -1);
        if ( s1!=s2 )  seq++;
        s1 = s2;

        // Option: style of output:
        cout << (v>0 ? '*' : (v<0?' ':'0') );  // for Walsh
//        cout << (v>0 ? '+' : (v<0?'-':' ') ); // for Haar
//        cout << (v ? '*' : ' ');  // for Reed-Muller and permutations
//        cout << f[k] << " ";

//        cout << setw(2) << (v); // debug
//        if ( v==0 )  cout << "  "; else cout << setw(2) << v;  // debug
//        cout << (v>0 ? '*' : (v<0?'-':' ') ); // debug

        if ( k<n-1 ) cout << ' ';
    }
    cout << " ]";
    cout << " (" << setw(2) << seq << ")"; // sequency

//    Type z =  norm(f, n);
////    Type z =  0;  for (ulong k=0; k<n; ++k)  if ( f[k]>0.001 ) z+=1;
//    cout << " |" << setw(8) << z << "|";  // norm
}
// -------------------------



void
func(double *f, ulong ldn, ulong x, double *g=0)
// The playground
{
    static int q = 1;
    ulong n = 1UL << ldn;

    TT( walsh_gray(f, ldn); );
    TT( grs_negate(f, n); );
    TT( revbin_permute(f, n); );

    if ( g )
    {
        TT( walsh_wal(g, ldn); );
    }

    x = 0; // avoid warning when unused
    n = 0; // avoid warning when unused
    q = 0; // used in macro TT()
}
// -------------------------

int
main(int argc, char **argv)
{
    ulong ldn = 5;
    if ( argc>1 )  ldn = atol(argv[1]);
    ulong  n = (1<<ldn);

    ulong q2 = 0;
    if ( argc>2 )  q2 = 1;

    double *f = new double[n];  null(f, n);
    double *g = 0;
    if ( q2 ) { g = new double[n];  null(g, n); }

    cout << "Operations:" << endl;
    for (ulong x=0; x<n; ++x)
    {
        null(f, n);  f[x] = 1;

        if ( g )  copy(f, g, n);

        func(f, ldn, x, g);

        prsgn(f, n, x, 0);

        double d;
        ulong i = test_delta(f, n, &d);
        if ( i<n )  cout << " d[" << setw(2) << i << "] = " << d;

        if ( g )  prsgn(g, n, x, 0);
//        if ( g )  cout << "  " << rms_diff(f, g, n);

        cout << endl;
    }

    return 0;
}
// -------------------------
