
#include "walsh/walshbasefunc.h"
#include "aux1/copy.h"
#include "aux1/symmetry.h"
#include "walsh/walshwak.h"

#include "walsh-all.h"  // for other transforms
#include "fxtiomanip.h"
#include "demo/nextarg.h"

#include <cmath>

//% Eigenvectors of the Walsh transform (Walsh-Kronecker basis).

void
print_vec(double *v, ulong n)
{
    cout << "[";
    for (ulong j=0; j<n; ++j)  cout << setw(2) << v[j] << " ";
    cout << "]" << endl;
}
// -------------------------


void
walsh_eigen(double *v, ulong ldn, ulong k)
// Eigenvectors of the Walsh transform (walsh_wak).
// Eigenvalues are +1 if k<n/2, else -1
{
    ulong n = 1UL << ldn;
    walsh_wak_basefunc(v, n, k);
    double d = sqrt(n);
    v[k] += (k<n/2 ? +d : -d);
}
// -------------------------


int
main(int argc, char **argv)
{
    ulong ldn = 4;
    NXARG(ldn, "Length 2**ldn Walsh transform.");

    ulong tq = 0;
    NXARG(tq, "Whether to print transformed vectors.");

    ulong n = 1UL << ldn;
    double *v = new double[n];
    for (ulong k=0; k<n; ++k)
    {
        cout << setw(2) << k << ":  ";
        walsh_eigen(v, ldn, k);
        print_vec(v, n);

        if ( tq )
        {
            walsh_wak(v, ldn);
            for (ulong j=0; j<n; ++j)  v[j] *= 1.0/sqrt(n);
            cout << "  :  ";
            print_vec(v, n);    
        }
        cout << endl;
    }

    return 0;
}
// -------------------------
