
#include "perm/permstar.h"

#include "aux0/swap.h"


perm_star::perm_star(ulong n)
    : n_(n)
{
#ifdef PERM_STAR_WITH_INVERSE
    a_ = new ulong[4*n_+1];
#else
    a_ = new ulong[3*n_+1];
#endif
    b_ = a_ + n_;
    c_ = b_ + n_;
#ifdef PERM_STAR_WITH_INVERSE
    ia_ = c_ + n_ + 1;
#endif
    first();
}
// -------------------------


perm_star::~perm_star()
{
    delete [] a_;
}
// -------------------------


void
perm_star::first()
{
    swp_ = n_ - 1;
    for (ulong k=0; k<n_; ++k)  a_[k] = b_[k] = k;
    for (ulong k=0; k<=n_; ++k)  c_[k] = 0;
#ifdef PERM_STAR_WITH_INVERSE
    for (ulong k=0; k<n_; ++k)  ia_[k] = k;
#endif
    idx_ = 0;
}
// -------------------------


ulong
perm_star::next()
{
    ulong k = 1;

    if ( c_[k] == k )
    {
        do
        {
            c_[k] = 0;
            ++k;
        }
        while ( c_[k] >= k );
    }

    if ( k == n_ )  first();
    else
    {
        ++c_[k];

        swp_ = b_[k];
        swap2(a_[0], a_[swp_]);
#ifdef PERM_STAR_WITH_INVERSE
        swap2(ia_[a_[0]], ia_[a_[swp_]]);
#endif

        ulong j = 1;
        --k;

        while ( j < k )  // < 0.18 iterations per call
        {
            swap2(b_[j], b_[k]);
            ++j;
            --k;
        }
        ++idx_;
    }

    return  idx_;
}
// -------------------------
